/* eslint-disable no-param-reassign */
import _ from 'lodash';
import { findTools } from '@appirio/appirio/lib/tools/tools';

const fs = require('fs');
const config = require('@appirio/appirio/lib/config/config');

const TOOL_CHECKER = 'TOOL_CHECKER';
const TOOL_VERSION_CHANGED = 'TOOL_VERSION_CHANGED';
const TOOL_INSTALL_START = 'TOOL_INSTALL_START';
const UPDATE_SETTING_START = 'UPDATE_SETTING_START';
const CONFIG_SETTING = 'CONFIG_SETTING';
const UPDATE_SETTING = 'UPDATE_SETTING';
const UPDATE_USER_SETTINGS = 'UPDATE_USER_SETTINGS';
const REPORT_ISSUE = 'REPORT_ISSUE';
const GET_USER_SETTING = 'GET_USER_SETTING';
const TOOL_INSTALLATION_DENIED = 'TOOL_INSTALLATION_DENIED';

function toolStatusChecker() {
  const listOfTools = findTools();
  return {
    type: TOOL_CHECKER,
    payload: listOfTools
  };
}

function ToolVersionUpdater(tool) {
  config.purge();
  let desktopAppCounter = config.hasUserConfig('desktopAppCounter') ? config.readUserConfig('desktopAppCounter') : undefined;
  if (tool.installationstatus === 'Not installed') {
    if (desktopAppCounter === undefined || desktopAppCounter[tool.id] === undefined) {
      tool.installationCounter = 1;
    } else {
      tool.installationCounter = desktopAppCounter[tool.id];
    }
  } else {
    if (desktopAppCounter === undefined) {
      desktopAppCounter = {};
    }
    if (tool.recommendation) {
      if (desktopAppCounter[tool.id] !== undefined) {
        desktopAppCounter[tool.id] += 1;
      } else {
        desktopAppCounter[tool.id] = 1;
      }
    } else {
      desktopAppCounter[tool.id] = 0;
    }
    tool.installationCounter = desktopAppCounter[tool.id];
    config.writeUserConfig('desktopAppCounter', desktopAppCounter);
  }
  return {
    type: TOOL_VERSION_CHANGED,
    payload: tool
  };
}

function ToolInstallStart(tool) {
  config.purge();
  let desktopAppCounter = config.hasUserConfig('desktopAppCounter') ? config.readUserConfig('desktopAppCounter') : undefined;
  if (desktopAppCounter === undefined) {
    desktopAppCounter = {};
  }
  if (desktopAppCounter[tool.id] === undefined) {
    desktopAppCounter[tool.id] = 0;
  }
  desktopAppCounter[tool.id] += 1;
  config.writeUserConfig('desktopAppCounter', desktopAppCounter);

  return {
    type: TOOL_INSTALL_START,
    payload: tool
  };
}

function ToolInstallationDenied(tool, installationError) {
  config.purge();
  let desktopAppCounter = config.hasUserConfig('desktopAppCounter') ? config.readUserConfig('desktopAppCounter') : undefined;
  if (tool.installationstatus === 'Not installed' || tool.installationstatus === 'Installing...') {
    if (desktopAppCounter === undefined || desktopAppCounter[tool.id] === undefined) {
      tool.installationCounter = 1;
    } else {
      tool.installationCounter = desktopAppCounter[tool.id];
    }
  } else {
    if (desktopAppCounter === undefined) {
      desktopAppCounter = {};
    }
    if (desktopAppCounter[tool.id] === undefined) {
      desktopAppCounter[tool.id] = 1;
    } else {
      desktopAppCounter[tool.id] += 1;
    }
    tool.installationCounter = desktopAppCounter[tool.id];
    config.writeUserConfig('desktopAppCounter', desktopAppCounter);
  }
  if (installationError !== undefined) {
    tool.installationError = installationError;
  }
  return {
    type: TOOL_INSTALLATION_DENIED,
    payload: tool
  };
}

function toolUpdateStart(tool, setting) {
  return {
    type: UPDATE_SETTING_START,
    payload: tool,
    settings: setting
  };
}

function setConfigSettings(tool, settings, initialState) {
  return {
    type: CONFIG_SETTING,
    payload: tool,
    setting: settings,
    initial: initialState
  };
}

function updateSetting(tool, setting) {
  return {
    type: UPDATE_SETTING,
    payload: tool,
    settings: setting
  };
}

function updateUserSetting(formData) {
  _.forOwn(formData, (settingValue, setting) => {
    config.writeUserConfig(setting, settingValue);
  });
  return {
    type: UPDATE_USER_SETTINGS,
    payload: formData
  };
}

function submitIssueToRollbar(formData, AppState) {
  return {
    type: REPORT_ISSUE,
    payload: formData,
    currentState: AppState
  };
}

function showModal(isShowingModal, toolToShowSettingsFor) {
  return {
    type: 'SHOW_MODAL',
    payload: isShowingModal,
    tool: toolToShowSettingsFor
  };
}

function showServicesModal(isShowingModal, serviceToShowSettingsFor) {
  return {
    type: 'SHOW_SERVICE_MODAL',
    payload: isShowingModal,
    service: serviceToShowSettingsFor
  };
}

function showNewProjectModal(isShowingModal) {
  return {
    type: 'SHOW_NEW_PROJECT_MODAL',
    payload: isShowingModal
  };
}

function showErrorModal(isShowingModal, error) {
  return {
    type: 'SHOW_ERROR_MODAL',
    payload: isShowingModal,
    error,
  };
}

function showProjectDetailModal(isShowingModal, project) {
  return {
    type: 'SHOW_PROJECT_DETAIL_MODAL',
    payload: isShowingModal,
    project,
  };
}

function projectArchive(currentArchiveStatus, projectsMap, targetProject) {
  config.purge();
  const projects = config.hasUserConfig('projects') ? config.readUserConfig('projects') : [];
  projectsMap = _.mapKeys(projects, 'path');
  projectsMap[targetProject.path].archived = !currentArchiveStatus;
  const projectsList = _.values(projectsMap);
  config.writeUserConfig('projects', projectsList);

  return {
    type: 'PROJECT_ARCHIVE',
    payload: !currentArchiveStatus,
    project: targetProject
  };
}

function showArchivedProjects(currentStatus, sortField) {
  config.purge();
  const projects = config.hasUserConfig('projects') ? config.readUserConfig('projects') : [];
  return {
    type: 'SHOW_ARCHIVE',
    payload: currentStatus,
    projects,
    sortField,
  };
}

function sortProjects(sortField, archivedProjects) {
  config.purge();
  const projects = config.hasUserConfig('projects') ? config.readUserConfig('projects') : [];
  return {
    type: 'SORT_PROJECTS',
    payload: sortField,
    projects,
    showArchivedProjects: archivedProjects,
  };
}

function refreshProjects(archivedProjects, sortField) {
  config.purge();
  const projects = config.hasUserConfig('projects') ? config.readUserConfig('projects') : [];
  const existingProjects = [];
  projects.map(project => {
    if (fs.existsSync(project.path)) {
      return existingProjects.push(project);
    }
  });
  config.writeUserConfig('projects', existingProjects);
  return {
    type: 'REFRESH_PROJECTS',
    payload: existingProjects,
    showArchivedProjects: archivedProjects,
    sortField,
  };
}

function tabClick(tabId) {
  return {
    type: 'TAB_CHANGED',
    payload: tabId
  };
}

function showHavingTroubleModal(isShowingModal, toolWhereInstallationFailed) {
  return {
    type: 'SHOW_HAVING_TROUBLE_MODAL',
    payload: isShowingModal,
    tool: toolWhereInstallationFailed
  };
}

module.exports = {
  TOOL_CHECKER,
  TOOL_VERSION_CHANGED,
  TOOL_INSTALL_START,
  UPDATE_SETTING_START,
  CONFIG_SETTING,
  UPDATE_SETTING,
  UPDATE_USER_SETTINGS,
  REPORT_ISSUE,
  GET_USER_SETTING,
  TOOL_INSTALLATION_DENIED,
  toolStatusChecker,
  ToolVersionUpdater,
  ToolInstallStart,
  toolUpdateStart,
  setConfigSettings,
  updateSetting,
  updateUserSetting,
  submitIssueToRollbar,
  ToolInstallationDenied,
  showModal,
  showServicesModal,
  showNewProjectModal,
  projectArchive,
  showArchivedProjects,
  sortProjects,
  showProjectDetailModal,
  refreshProjects,
  tabClick,
  showHavingTroubleModal,
  showErrorModal,
};
