import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { remote } from 'electron';
import { HashRouter, Route, Switch } from 'react-router-dom';
import ReactModal from 'react-modal';
import ReactHtmlParser from 'react-html-parser';
import { mapOfTools, installADXOnAppReload } from '@appirio/appirio/lib/tools/tools';
import { POWERSHELL_ENABLED } from '@appirio/appirio/lib/util/powershell';
import rollbar from '@appirio/appirio/lib/notify/rollbar';
import ToolsSection from '../containers/tools_list';
import ProjectSection from '../containers/project-section';
import Services from '../containers/services';
import UserSettings from '../containers/user_settings';
import NewProject from '../containers/new_project';
import ProjectDetail from '../containers/project_detail';
import ReportIssue from '../containers/report_issue';
import NavBar from '../containers/NavBar';

const appVersion = `${remote.app.getName()} v${remote.app.getVersion()}`;

export default class App extends Component {
  componentDidMount() {
    const error = `
    <span>
      Cannot find Powershell path in the PATH environment variable. Please add it before proceeding.
      You can follow the below steps to add it to the PATH:
      <ol>
        <li>
          Look for the version folder of <strong>Windows Powershell</strong> on your system. The PowerShell executable is likely to be at this location:
          <strong>'C:\\Windows\\System32\\WindowsPowerShell\\v1.1'</strong>.
        </li>
        <li>
          Go to your environment variables and add this path to the <strong>System PATH</strong> variable, and click 'OK'.
        </li>
        <li>
          You may need to reboot your system.
        </li>
      </ol>
    </span>
    `;
    if (POWERSHELL_ENABLED === false) {
      const errMessage = 'Powershell not found in Windows PATH';
      const payload = {
        context: 'powershellPathNotFound',
        action: 'loadError',
      };
      rollbar.sendMessage(errMessage, true, payload);
      this.props.actions.showErrorModal(true, error);
    }
    if (this.props.user.userInfo !== 'Hey, Who are you?') {
      this.props.actions.ToolInstallStart(mapOfTools.adx);
      installADXOnAppReload()
        .then((installResult) => {
          return this.props.actions.ToolVersionUpdater(mapOfTools.adx);
        })
        .catch(err => {
          console.log(err);
        });
    }
  }

  quitApp() {
    const window = remote.getCurrentWindow();
    window.close();
  }

  render() {
    const customStyles = {
      content: {
        border: '5px solid gray',
        background: 'white',
        overflow: 'auto',
        WebkitOverflowScrolling: 'touch',
        borderRadius: '4px',
        outline: 'none',
        padding: '1rem',
        top: '50%',
        left: '50%',
        right: 'auto',
        bottom: 'auto',
        marginRight: '-50%',
        width: '50%',
        transform: 'translate(-50%, -50%)',
      }
    };

    return (
      <HashRouter>
        <div className="container-fluid" >
          <NavBar />
          {
            this.props.isShowingModal.showErrorModal &&
            <ReactModal
              isOpen={this.props.isShowingModal.showErrorModal}
              contentLabel="Error Modal"
              style={customStyles}
            >
              <div>
                <nav className="navbar navbar-dark errorModal">
                  <span style={{
                    color: 'white', fontSize: '25px', verticalAlign: 'text-top', paddingLeft: '2%'
                  }}
                  >
                    Oops!
                  </span>
                </nav>
                <div style={{ color: '#8b0000' }}>
                  {ReactHtmlParser(this.props.isShowingModal.error)}
                </div>
                <div style={{
                  textAlign: 'center',
                }}
                >
                  <button onClick={() => this.quitApp()} className="quitAppButton">OK</button>
                </div>
              </div>
            </ReactModal>
          }
          <div className="row mainContainer" >
            <Switch>
              <Route path="/tools" component={ToolsSection} />
              <Route path="/services" component={Services} />
              <Route path="/usersettings" component={UserSettings} />
              <Route path="/newproject" component={NewProject} />
              <Route path="/reportissue" component={ReportIssue} />
              <Route path="/projectdetail/:name" component={ProjectDetail} />
              <Route path="/" component={this.props.user.userInfo !== 'Hey, Who are you?' && !this.props.isShowingModal.showErrorModal ? ProjectSection : UserSettings} />
            </Switch>
            <div className="footer">{appVersion}</div>
          </div>
        </div >
      </HashRouter>
    );
  }
}

App.propTypes = {
  user: PropTypes.objectOf(PropTypes.any).isRequired,
  actions: PropTypes.objectOf(PropTypes.any).isRequired,
  isShowingModal: PropTypes.objectOf(PropTypes.any).isRequired,
};
