/* eslint global-require: 0, flowtype-errors/show-errors: 0 */

/**
 * This module executes inside of electron's main process. You can start
 * electron renderer process from here and communicate with the other processes
 * through IPC.
 *
 * When running `npm run build` or `npm run build-main`, this file is compiled to
 * `./app/main.prod.js` using webpack. This gives us some performance wins.
 *
 * @flow
 */
import {
  app,
  BrowserWindow,
  ipcMain,
  Notification
} from 'electron';
import MenuBuilder from './menu';

const { autoUpdater } = require('electron-updater');
autoUpdater.logger = require('electron-log');

autoUpdater.logger.transports.file.level = 'info';
let alwaysNotify = false;
const checkForUpdatesAndNotify = (manualUpdate) => {
  alwaysNotify = (manualUpdate === true);
  autoUpdater.checkForUpdatesAndNotify();
};

// By default electron-log writes logs to the following locations:
// on Linux: ~/.config/<app name>/log.log
// on OS X: ~/Library/Logs/<app name>/log.log
// on Windows: %USERPROFILE%\AppData\Roaming\<app name>\log.log

let mainWindow = null;

if (process.env.NODE_ENV === 'production') {
  const sourceMapSupport = require('source-map-support');
  sourceMapSupport.install();
}

if (process.env.NODE_ENV === 'development' || process.env.DEBUG_PROD === 'true') {
  require('electron-debug')();
  const path = require('path');
  const p = path.join(__dirname, '..', 'app', 'node_modules');
  require('module').globalPaths.push(p);
}

const installExtensions = async () => {
  const installer = require('electron-devtools-installer');
  const forceDownload = !!process.env.UPGRADE_EXTENSIONS;
  const extensions = [
    'REACT_DEVELOPER_TOOLS',
    'REDUX_DEVTOOLS'
  ];

  return Promise
    .all(extensions.map(name => installer.default(installer[name], forceDownload)))
    .catch(console.log);
};

/**
 * Add event listeners...
 */

app.on('window-all-closed', () => {
  // Respect the OSX convention of having the application in memory even
  // after all windows have been closed
  if (process.platform !== 'darwin') {
    app.quit();
  }
});

app.on('ready', async () => {
  if (process.env.NODE_ENV === 'development' || process.env.DEBUG_PROD === 'true') {
    await installExtensions();
  }

  mainWindow = new BrowserWindow({
    show: false,
    width: 1350,
    height: 800
  });

  mainWindow.loadURL(`file://${__dirname}/app.html#v${app.getVersion()}`);

  // @TODO: Use 'ready-to-show' event
  //        https://github.com/electron/electron/blob/master/docs/api/browser-window.md#using-ready-to-show-event
  mainWindow.webContents.on('did-finish-load', () => {
    if (!mainWindow) {
      throw new Error('"mainWindow" is not defined');
    }
    checkForUpdatesAndNotify();
    mainWindow.show();
    mainWindow.focus();
  });

  mainWindow.on('closed', () => {
    mainWindow = null;
    app.quit();
  });

  const menuBuilder = new MenuBuilder(mainWindow, checkForUpdatesAndNotify);
  menuBuilder.buildMenu();
});

/*  AUTO-UPDATER DEBUGGING */
// when the update is ready, notify the BrowserWindow
autoUpdater.on('update-downloaded', (info) => {
  mainWindow.webContents.send('updateReady');
});
// autoUpdater.on('checking-for-update', () => {
//   sendStatusToWindow('Checking for update...');
// })
// autoUpdater.on('update-available', (info) => {
//   sendStatusToWindow('Update available.');
// })
autoUpdater.on('update-not-available', (info) => {
  // TODO: We send this status, but we are not displaying it anywhere
  sendStatusToWindow('No update available.');
});
// autoUpdater.on('error', (err) => {
//   sendStatusToWindow('Error in auto-updater. ' + err);
// })
// autoUpdater.on('download-progress', (progressObj) => {
//   let log_message = "Download speed: " + progressObj.bytesPerSecond;
//   log_message = log_message + ' - Downloaded ' + progressObj.percent + '%';
//   log_message = log_message + ' (' + progressObj.transferred + "/" + progressObj.total + ')';
//   sendStatusToWindow(log_message);
// })

function sendStatusToWindow(text) {
  // We can display messages in the user interface by adding code similar
  // to this: https://github.com/iffy/electron-updater-example/blob/610a3bf2b27dde8efbd772ffcec8f66185a626db/version.html
  console.log(text);
  mainWindow.webContents.send('versionMessage', text);
  if (alwaysNotify) {
    const message = new Notification(text, {
      body: text
    });
  }
}

const electron = require('electron');

const ipc = electron.ipcRenderer;
const dialog = electron.dialog;
ipcMain.on('open-file-dialog-for-file', (event) => {
  dialog.showOpenDialog({
    properties: ['openDirectory']
  }, (files) => {
    if (files) {
      event.sender.send('selected-file', files[0]);
    } else event.sender.send('selected-file', undefined);
  });
});
