// @flow
import {
  app,
  Menu,
  shell,
  BrowserWindow
} from 'electron';
import { betaUser } from '@appirio/appirio/lib/config/config';

const devMode = betaUser() || process.env.NODE_ENV === 'development' || process.env.DEBUG_PROD === 'true';

export default class MenuBuilder {
  mainWindow: BrowserWindow;

  constructor(mainWindow: BrowserWindow, checkForUpdates) {
    this.mainWindow = mainWindow;
    this.checkForUpdates = checkForUpdates;
  }

  buildMenu() {
    if (devMode) {
      this.setupDevelopmentEnvironment();
      this.mainWindow.openDevTools();
    }
    const template = this.buildTemplate();
    const menu = Menu.buildFromTemplate(template);
    Menu.setApplicationMenu(menu);

    return menu;
  }

  setupDevelopmentEnvironment() {

    this.mainWindow.webContents.on('context-menu', (e, props) => {
      const {
        x,
        y
      } = props;

      Menu
        .buildFromTemplate([{
          label: 'Inspect element',
          click: () => {
            this.mainWindow.inspectElement(x, y);
          }
        }])
        .popup(this.mainWindow);
    });
  }

  buildTemplate() {
    const subMenuAbout = (process.platform === 'darwin') ? {
      label: 'AppirioDX',
      submenu: [
        { label: 'About AppirioDX', selector: 'orderFrontStandardAboutPanel:' },
        { type: 'separator' },
        { label: 'Hide AppirioDX', accelerator: 'Command+H', selector: 'hide:' },
        { label: 'Hide Others', accelerator: 'Command+Shift+H', selector: 'hideOtherApplications:' },
        { label: 'Show All', selector: 'unhideAllApplications:' },
        { type: 'separator' },
        { label: 'Quit', accelerator: 'Command+Q', click: () => { app.quit(); } }
      ]
    } : {
      label: '&File',
      submenu: [
        { label: '&Open', accelerator: 'Ctrl+O' },
        {
          label: '&Close',
          accelerator: 'Ctrl+W',
          click: () => {
            this.mainWindow.close();
          }
        }
      ]
    };

    const subMenuEdit = {
      label: 'Edit',
      submenu: [
        { label: 'Undo', accelerator: 'CmdOrCtrl+Z', selector: 'undo:' },
        { label: 'Redo', accelerator: 'Shift+CmdOrCtrl+Z', selector: 'redo:' },
        { type: 'separator' },
        { label: 'Cut', accelerator: 'CmdOrCtrl+X', selector: 'cut:' },
        { label: 'Copy', accelerator: 'CmdOrCtrl+C', selector: 'copy:' },
        { label: 'Paste', accelerator: 'CmdOrCtrl+V', selector: 'paste:' },
        { label: 'Select All', accelerator: 'CmdOrCtrl+A', selector: 'selectAll:' }
      ]
    };

    const subMenuView = {
      label: '&View',
      submenu: []
    };
    subMenuView.submenu.push({
      label: 'Toggle &Developer Tools',
      accelerator: 'Alt+CmdOrCtrl+I',
      click: () => {
        this.mainWindow.toggleDevTools();
      }
    });

    const subMenuWindow = {
      label: '&Window',
      submenu: [{
        label: 'Minimize',
        accelerator: 'Command+M',
        selector: 'performMiniaturize:'
      }, {
        label: 'Reload',
        accelerator: 'CmdOrCtrl+R',
        click: (item, focusedWindow) => {
          if (focusedWindow) {
            focusedWindow.reload();
          }
        }
      }]
    };

    const subMenuHelp = {
      role: 'help',
      submenu: [{
          label: 'Help',
          click() {
            shell.openExternal('https://dx.appirio.com/docs/');
          }
        },
        {
          label: 'Report an issue',
          click() { shell.openExternal('https://docs.google.com/forms/d/e/1FAIpQLSc-gttyPqNGjAgerHRPxYCGTD1XB0oZhC38Odz-SN_CvEprVw/viewform?usp=sf_link') }
        },
        {
          label: 'Check for Updates',
          click: () => {
            this.checkForUpdates(true);
          }
        }
      ]
    };

    return [
      subMenuAbout,
      subMenuEdit,
      subMenuView,
      subMenuWindow,
      subMenuHelp
    ];
  }
}
